﻿using Bitlog.Api;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.IO;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading.Tasks;

namespace BitlogErpConsole
{
    class Program
    {
        private static ApiConfig _settings;

        static async Task Main(string[] args)
        {
            Console.WriteLine("Bitlog ERP client");
            Console.WriteLine();

            bool exit = false;
            string data = null;
            string command = "l";

            do
            {
                try
                {
                    command = command.Trim();

                    if (command.Length > 1)
                    {
                        data = command.Substring(1).Trim();
                        command = command.Substring(0, 1);
                    }

                    switch (command.ToLower())
                    {
                        case "h":
                            ShowHelp();
                            break;

                        case "l":
                            LoadSettings();
                            Console.WriteLine();
                          
                            ShowSettings();
                            Console.WriteLine();
                            
                            ShowHelp();
                            break;

                        case "v":
                            SaveSettings();
                            break;

                        case "s":
                            ShowSettings();
                            break;

                        case "t":
                            await GetNewAccessTokenAsync();
                            break;

                        case "g":
                            await GetWarehousesAsync();
                            break;

                        case "p":
                            await PostWarehousesAsync();
                            break;

                        case "u":
                            await UpdateWarehouseAsync();
                            break;

                        case "d":
                            await DeleteWarehouseAsync();
                            break;

                        case "x":
                            exit = true;
                            break;
                    }
                }

                catch (Exception ex)
                {
                    ShowError(ex);
                }

                if (!exit)
                {
                    Console.WriteLine();
                    Console.Write(">");

                    data = null;
                    command = Console.ReadLine();
                }
            }
            while (!exit);

            System.Threading.Thread.Sleep(300);
        }

        private static void LoadSettings()
        {
            Console.WriteLine("Loading settings...");
            
            _settings = JsonConvert.DeserializeObject<ApiConfig>(File.ReadAllText("bitlog.json"));
        }
        private static void SaveSettings()
        {
            Console.WriteLine("Saving settings...");

            File.WriteAllText("bitlog.json", JsonConvert.SerializeObject(_settings));
        }
        private static void ShowSettings()
        {
            Console.WriteLine($" URL:           {_settings.url}");
            Console.WriteLine($" Client Id:     {_settings.clientId}");
            Console.Write(" Token:         ");

            if (!string.IsNullOrEmpty(_settings.accessToken))
                Console.WriteLine("(defined)");
            else
                Console.WriteLine("(none)");
        }
        private static void ShowHelp()
        {
            Console.WriteLine(":: Settings menu ::");
            Console.WriteLine("  l - Load settings");
            Console.WriteLine("  v - Save settings");
            Console.WriteLine("  s - Show settings");
            Console.WriteLine("  t - Retrieve access token");
            Console.WriteLine();
            Console.WriteLine(":: Testing menu ::");
            Console.WriteLine("  g - Get all warehouses");
            Console.WriteLine("  p - Post warehouse");
            Console.WriteLine("  u - Update warehouse");
            Console.WriteLine("  d - Delete warehouse");
            Console.WriteLine();
            Console.WriteLine("  h - Help");
            Console.WriteLine("  x - Exit");
        }
        private static void ShowError(Exception ex)
        {
            Console.WriteLine();
            Console.WriteLine(" Error: " + ex.Message);

            if (ex.InnerException != null)
                Console.WriteLine("   Inner: " + ex.InnerException.Message);
        }

        private static async Task GetNewAccessTokenAsync()
        {
            Console.WriteLine("Retrieving new access token...");

            var handler = new HttpClientHandler();
            var client = new HttpClient(handler);
            var header = Convert.ToBase64String(Encoding.UTF8.GetBytes($"{_settings.clientId}:{_settings.clientSecret}"));

            client.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Basic", header);

            // Create a dictionary which contains the request form data, here we need to set
            // the username, password and grant_type as shown below
            var body = new Dictionary<string, string>
                    {
                    {"grant_type", "password"},
                    {"username", _settings.userName},
                    {"password", _settings.password},
                    };

            //Make a Post request by invoking the PostAsync method on the client object as shown below
            var uri = new Uri(_settings.url);

            var response = await client.PostAsync(new Uri(uri, "token"), new FormUrlEncodedContent(body));

            response.EnsureSuccessStatusCode();

            var content = await response.Content.ReadAsStringAsync();

            var token = JsonConvert.DeserializeObject<Token>(content);

            Console.WriteLine($"Token: {token.AccessToken}");

            _settings.accessToken = token.AccessToken;
        }

        private static async Task GetWarehousesAsync()
        {
            Console.WriteLine("Retrieving warehouses...");
          
            using (var hc = new SecureHttpClient(_settings.accessToken))
            {
                var tc = new WarehouseClient(_settings.url, hc);

                var items = await tc.GetWarehousesAsync();

                foreach (var item in items)
                {
                    Console.WriteLine($" {item.Code,-10} {item.Name}");
                }
            }
     
            Console.WriteLine("Success");
        }
        private static async Task PostWarehousesAsync()
        {
            Console.WriteLine("Posting warehouse");

            using (var hc = new SecureHttpClient(_settings.accessToken))
            {
                var tc = new WarehouseClient(_settings.url, hc);

                var m = new WarehouseModel();

                m.Code = "99";
                m.Name = "Spare parts";

                await tc.PostWarehouseAsync(m);
            }

            Console.WriteLine("Success");
        }
        private static async Task UpdateWarehouseAsync()
        {
            Console.WriteLine("Updating warehouse");

            using (var hc = new SecureHttpClient(_settings.accessToken))
            {
                var tc = new WarehouseClient(_settings.url, hc);

                var m = new WarehouseModel();

                m.Code = "99";
                m.Name = "Cars";

                await tc.PutWarehouseAsync(m.Code, m);
            }

            Console.WriteLine("Success");
        }
        private static async Task DeleteWarehouseAsync()
        {
            Console.WriteLine("Deleting warehouse");
            
            using (var hc = new SecureHttpClient(_settings.accessToken))
            {
                var tc = new WarehouseClient(_settings.url, hc);

                await tc.DeleteWarehouseAsync("99");
            }

            Console.WriteLine("Success");
        }
    }
}
